SystemOrganization addCategory: #'Math-AccuracyFramework'!

Object subclass: #Accuracy
	instanceVariableNames: 'arguments parameters names results iterations numberOfResults numberOfParameters aStream dataTree'
	classVariableNames: 'DecimalPlaces'
	poolDictionaries: ''
	category: 'Math-AccuracyFramework'!

!Accuracy commentStamp: '<historical>' prior: 0!
Accuracy is a framework for testing the numerical accuracy of the results of methods.
!

!Accuracy class methodsFor: 'accessing' stamp: 'WernerKassens 8/10/2014 16:35'!
decimalPlaces
^DecimalPlaces! !

!Accuracy class methodsFor: 'accessing' stamp: 'WernerKassens 7/31/2014 11:24'!
decimalPlaces: anInteger 
^DecimalPlaces:=anInteger ! !

!Accuracy class methodsFor: 'util' stamp: 'WernerKassens 8/8/2014 12:14'!
floatAsShortString: aFloat
^self floatAsShortString: aFloat digitCount: DecimalPlaces! !

!Accuracy class methodsFor: 'util' stamp: 'WernerKassens 9/11/2014 09:46'!
floatAsShortString: aFloat digitCount: digitCount 
"essentially copied fromFloat>>absPrintOn:base:digitCount:"
	| fuzz x exp q fBase scale logScale xi aStream posFloat |
	aFloat =0 ifTrue:[^'0.0'].
	aStream:=WriteStream on:''.
	posFloat :=aFloat <0 
		ifTrue:[aStream nextPut: $-.aFloat negated]
		ifFalse:[aFloat ].
	posFloat isInfinite ifTrue: [aStream nextPutAll: 'Infinity'.
		^ aStream contents].
	fBase := 10.0.
	"x is myself normalized to [1.0, fBase), exp is my exponent"
	exp := 
		posFloat  < 1.0
			ifTrue: [posFloat reciprocalFloorLog: fBase]
			ifFalse: [posFloat floorLog: fBase].
	scale := 1.0.
	logScale := 0.
	[(x := fBase raisedTo: (exp + logScale)) = 0]
		whileTrue:
			[scale := scale * fBase.
			logScale := logScale + 1].
	x := posFloat * scale / x.
	fuzz := fBase raisedTo: 1 - digitCount.
	"round the last digit to be printed"
	x := 0.5 * fuzz + x.
	x >= fBase
		ifTrue: 
			["check if rounding has unnormalized x"
			x := x / fBase.
			exp := exp + 1].
	(exp < (digitCount-1) and: [exp > (-2) ])
		ifTrue: 
			["decimal notation"
			q := 0.
			exp < 0 ifTrue: [1 to: 1 - exp do: [:i | aStream nextPut: ('0.' at:i)]]]
		ifFalse: 
			["scientific notation"
			q := exp.
			exp := 0].
	[x >= fuzz]
		whileTrue: 
			["use fuzz to track significance"
			xi := x asInteger.
			aStream nextPut: (Character digitValue: xi).
			x := x - xi asFloat * fBase.
			fuzz := fuzz * fBase.
			exp := exp - 1.
			exp = -1 ifTrue: [aStream nextPut: $.]].
	[exp >= -1]
		whileTrue: 
			[aStream nextPut: $0.
			exp := exp - 1.
			exp = -1 ifTrue: [aStream nextPut: $.]].
	q ~= 0
		ifTrue: 
			[aStream nextPut: $e.
			q printOn: aStream].
	^ aStream contents! !

!Accuracy class methodsFor: 'class initialization' stamp: 'WernerKassens 8/30/2014 14:27'!
initialize 
DecimalPlaces ifNil: [DecimalPlaces :=3].! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/11/2014 09:02'!
argument
|r|
r:=self argumentAt: self findKey.
numberOfResults ifNotNilDo: [ :rn|r:=r at:rn ].
^r.! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/13/2014 18:30'!
argument: anArgument
arguments at: self findKey put: (anArgument ifNil: [ nil ] ifNotNilDo: [:a| self asArray: a]).
^anArgument 
! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/9/2014 18:10'!
argumentAt: aName
|ar|
ar :=arguments at: aName ifAbsent: [arguments at:  'AllTheRest' ifAbsent: [ nil ]].
	(ar isCollection and: [ar isEmpty])ifTrue:[ar:=nil].
^ar.! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 9/4/2014 16:32'!
asArray: aCol 
^(aCol isCollection and: [aCol isSequenceable and:[aCol isString not] ])  ifTrue: [aCol asArray] ifFalse:[{aCol}]! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 9/3/2014 11:28'!
calcDeviations:aValue in: aCol max: aBoolean
|c|
c:=self extremeCollection: aCol max:aBoolean .
c:=aCol inject: c into: [:a :b| a with: b collect: [:a1 :b1| 
		aBoolean ifTrue: [ a1 max: b1 ] ifFalse: [a1 min: b1]]].
^c with: aValue collect:[:rr :r|self calcErrorOf: r realResult: rr ] .! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 8/6/2014 16:58'!
calcErrorOf: aResult realResult: aRResult
^aResult = 0 
	ifTrue: [aRResult =0 
		ifTrue: [0] 
		ifFalse:[aRResult >0 
			ifTrue:[ Float infinity]
			ifFalse:[Float infinity negated ]]] 
	ifFalse:[100.0*(aRResult-aResult) /aResult ]! !

!Accuracy methodsFor: 'running' stamp: 'WernerKassens 8/31/2014 09:44'!
calcResult: aName tree: aTree
	| aResult c|
	c := self performCheck: aName.
	self tree: aTree type: 'result' data: c .
	aResult := c average.
	self ifSeveralterations: [aStream <<'mean '] .
	(self format: aResult type: 'result' postfix:nil tree: aTree)space.
	self ifSeveralterations: [ self streamDeviationsOfResult: aResult inCollection: c tree: aTree].
	^ aResult ! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 9/11/2014 09:20'!
dataTree
^dataTree! !

!Accuracy methodsFor: 'running' stamp: 'WernerKassens 9/10/2014 21:11'!
displayProgress
'Checking'	displayProgressFrom: 1 to: names size during: [ :bar | names do: [ :n | 
					bar label: 'Checking ' , n.
					bar increment.
					self streamTest: n ] ].
^self! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 8/10/2014 02:12'!
extractFromResults: theResults which: num onlyOne: aBoolean
	| aResult |
	numberOfResults := aBoolean 
		ifTrue: [ aResult := theResults. nil ]
		ifFalse: [ aResult := theResults at: num.	num ].
	^ aResult! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 8/10/2014 01:28'!
extremeCollection:acol max:aBoolean 
|c|
c:=acol first.
c:=c isCollection ifTrue: [c size] ifFalse:[1]. 
^Array new: c withAll: (aBoolean ifTrue: [ Float infinity negated] ifFalse: [Float infinity])! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 9/3/2014 10:13'!
findKey
|s m|
s:=thisContext sender.
m :=s sender method selector .
^(names detect: [:n| m asString endsWith: n] ifNone: [nil]) ifNil: [m='initialize' 
	ifTrue: ['AllTheRest'] 
	ifFalse:[m:=s method selector asString. self error: m,' called in wrong context'] ]! !

!Accuracy methodsFor: 'printing' stamp: 'WernerKassens 8/10/2014 00:47'!
format: aCollection
|col|
col:= self asArray: aCollection.
^col collect: [ :a | a isNumber 
		ifTrue: [ self class floatAsShortString:a]
		ifFalse: [ a ] ].
! !

!Accuracy methodsFor: 'printing' stamp: 'WernerKassens 9/2/2014 22:48'!
format: aCollection type: aString postfix: pf
	| c |
	c := self format: aCollection.
	aStream << aString <<': '<< (c joinUsing: (pf ifNil: [ ' , ' ] ifNotNil: [ pf , ' , ' ])).
	pf ifNotNil: [ aStream << pf ].
	^aStream space.! !

!Accuracy methodsFor: 'printing' stamp: 'WernerKassens 9/2/2014 22:42'!
format: aCollection type: aString postfix: pf tree: aTree
	self format: aCollection type: aString postfix: pf.
	aTree at:aString put: (aCollection size=1 ifTrue:[aCollection first] ifFalse:[aCollection]).
	^aStream ! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 7/31/2014 14:23'!
ifSeveralterations: aBlock
iterations >1 ifTrue:[^aBlock value].
! !

!Accuracy methodsFor: 'initialize-release' stamp: 'WernerKassens 7/31/2014 15:49'!
initNames
	names := (self class allSelectorsBelow: Object)
		select: [ :s | s beginsWith: #check ]
		thenCollect: [ :s | s copyFrom: 6 to: s size ].
	names := names asOrderedCollection sort! !

!Accuracy methodsFor: 'initialize-release' stamp: 'WernerKassens 7/31/2014 11:21'!
initRest: aName 
|i|
i := ('initialize', aName) asSymbol.
(self respondsTo: i) ifTrue: [ self perform: i].! !

!Accuracy methodsFor: 'initialize-release' stamp: 'WernerKassens 9/11/2014 09:21'!
initialize
	self initNames.
	parameters := Dictionary new.
	arguments := Dictionary new.
	results := Dictionary new.
	aStream := WriteStream with: ''.
	iterations := 1.
	dataTree := KeyedTree new .
	names do: [ :n | self initRest: n ]! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/1/2014 08:03'!
iterations: anInteger 
anInteger <1 ifTrue:[^iterations].
^iterations :=anInteger ! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/30/2014 18:22'!
numberOfDifferentParametersAt: aname
"parameters have to be in a collection, even a single parameter.
never returns 0"
|no|
no := parameters at: aname  ifAbsent: [parameters at: 'AllTheRest' ifAbsent: [^1]].
no isCollection ifFalse: [^1].
^no size max: 1! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/5/2014 17:12'!
numberOfDifferentResultsAt: aname
|no|
no := self resultsAt: aname.
no isCollection ifFalse:[no:={no}].
^(no first) isCollection ifTrue: [ no size ] ifFalse: [ 1 ]! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/11/2014 08:59'!
parameter
|r|
r:=self parameterAt: self findKey .
numberOfParameters ifNotNilDo: [ :rn|r:=r at:rn ].
^r.
! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/31/2014 08:26'!
parameter: aParameter
(aParameter isArray or: [aParameter isNil])ifFalse:[self error: 'parameter must be an Array'].
(aParameter isArray and: [aParameter size=1]) ifTrue:  [ self error: 'parameter of size 1 is not possible' ].
^parameters  at: self findKey put: aParameter.
! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/11/2014 15:32'!
parameterAt: aName
|par|
par :=parameters at: aName ifAbsent: [parameters at:  'AllTheRest' ifAbsent: [ nil ]].
	(par isCollection and: [par isEmpty])ifTrue:[par:=nil].
^par! !

!Accuracy methodsFor: 'running' stamp: 'WernerKassens 9/10/2014 21:13'!
performCheck: aName
	^ (1 to: iterations)
		collect: [ :i | [ Processor yield. self setUp. self asArray: (self perform: ('check' , aName) asSymbol) ]
				ensure: [ self tearDown ]]! !

!Accuracy methodsFor: 'printing' stamp: 'WernerKassens 9/11/2014 09:26'!
printOn: aStream1
super printOn: aStream1.
self report  
	ifEmpty:[aStream1 nextPutAll: ' ()'] 
	ifNotEmptyDo:[:c|
		aStream1 nextPutAll: ' (
'; 		nextPutAll: c; 
		nextPutAll:' )'].! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 9/3/2014 10:34'!
putExpectedResult: anExpectedResult totree: aTree
	| e |
	e := anExpectedResult.
	[ (aTree at: e ifPresent: [ Array new: (e size + 1)])
		ifNil: [ ^aTree at: e put:  KeyedTree new ] 
		ifNotNilDo: [:a|
			a replaceFrom: 1 to: e size with: e startingAt: 1; 
			  at: a size put: $+.
			e:=a.
			false ] ] whileFalse.! !

!Accuracy methodsFor: 'printing' stamp: 'WernerKassens 9/11/2014 09:10'!
report
^aStream contents  
! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/11/2014 15:42'!
result: aResult
results at: self findKey put: (self asArray: aResult).
^aResult 
! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 8/1/2014 04:00'!
resultsAt: aName
^results at: aName ifAbsent: [results at: 'AllTheRest' ifAbsent:[nil]].
! !

!Accuracy methodsFor: 'accessing' stamp: 'WernerKassens 9/4/2014 16:59'!
resultsKeyFor: aName AtPosition: anInteger 
"utility to construct the key to look for in docu. see testResultsKeyAtPosition."
|aResults repetitions key result|
aResults := self resultsAt: aName.
key := aResults at:anInteger.
key isArray ifFalse:[^aResults ].
repetitions :=0.
aResults from:1 to: anInteger-1 do:[:i| i=key ifTrue:[repetitions:=repetitions+1]].
result := (Array new: (key size + repetitions))replaceFrom: 1 to: key size with: key startingAt: 1.
(key size+1) to: result size do:[:i|result at:i put: $+]. 
^result
! !

!Accuracy methodsFor: 'running' stamp: 'WernerKassens 9/11/2014 09:22'!
run
	aStream reset;
		<< 'Report for: ';
		<< self class name;
		cr.
	self ifSeveralterations: [ aStream << 'iterations: '	<< iterations; cr ].
	(self tree: dataTree removeAll type: self class name data: 'names')
		at: 'iterations' put: iterations;
		at: 'names' put: (self tree: KeyedTree new type: 'names' data: names).
	self displayProgress.
	^ aStream contents! !

!Accuracy methodsFor: 'running' stamp: 'WernerKassens 9/1/2014 09:24'!
setUp
	"It is the subclass' responsibility to set up the necessary environment for a check"
^self! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 8/30/2014 11:02'!
severalChecksOn: aName with: anExpectedResult tree: aTree
	| aResult  error |
	aResult := self calcResult: aName tree: aTree .
	error := aResult isCollection
		ifTrue: [ aResult with: anExpectedResult collect: [ :r :e | self calcErrorOf: r realResult: e ] ]
		ifFalse: [ self calcErrorOf: aResult realResult: anExpectedResult ].
	(self format: error type: 'error' postfix: '%' tree: aTree) cr; cr.
! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 8/30/2014 10:53'!
streamArgumentsAt: anInteger for: aName tree:aTree
|ar|
	ar :=self argumentAt: aName.
	ar  ifNil:[aStream cr]
	   ifNotNilDo: [ :a | 
		(self format:  (a at: anInteger) type: 'arguments' postfix:nil tree: aTree) cr]! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 8/30/2014 11:34'!
streamDeviationsOfResult: aResult inCollection: aCollection tree: aTree
|sd| 
self format: (self calcDeviations: aResult in: aCollection max: true) type:'+dev' postfix:'%'tree: aTree.
(self format: (self calcDeviations: aResult in: aCollection max: false) type: '-dev' postfix:'%'tree: aTree) cr.
sd:=aCollection first isCollection 
	ifTrue: [(1 to: aCollection first size)collect:[:k| (aCollection collect:[:j| j at: k ])stdev ]. ] 
	ifFalse:[aCollection stdev] .
(self format:  sd type: 'standard deviation' postfix:nil tree: aTree) cr.! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 9/2/2014 18:11'!
streamResultsFor: aName tree: aTree single: oneResult
	| anExpectedResult aNewTree |
	1 to: (self numberOfDifferentResultsAt: aName) do: [ :num | 
		anExpectedResult := self extractFromResults: (self resultsAt: aName) which: num onlyOne: oneResult.
		aNewTree := oneResult ifTrue: [ aTree ] ifFalse: [ self putExpectedResult: anExpectedResult totree: aTree ].
		self format: anExpectedResult type: 'expected result' postfix: nil tree: aNewTree.
		self streamArgumentsAt: num for: aName tree: aNewTree.
		self severalChecksOn: aName with: anExpectedResult tree: aNewTree ]! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 9/11/2014 09:23'!
streamTest: aName
	| pno namesTree subTree oneResult|
	namesTree := (dataTree at: 'names') at: aName put: (subTree := KeyedTree new).
	pno := self numberOfDifferentParametersAt: aName.
	oneResult := (self numberOfDifferentResultsAt: aName) = 1.
	aStream << 'test '	<< aName;	cr.
	1 to: pno do: [ :parameterNo | 
		pno > 1 ifTrue: [ subTree:=self streamTest: aName withParameter: parameterNo tree: namesTree ].
		oneResult ifFalse: [ self tree: subTree type: 'expected result' data: (self resultsAt: aName) ].
		self streamResultsFor: aName tree: subTree single: oneResult ] ! !

!Accuracy methodsFor: 'streaming' stamp: 'WernerKassens 9/2/2014 22:49'!
streamTest: aName withParameter: parameterNo tree: d
	| aParameter tree |
	self tree: d type: 'parameter' data: (aParameter := self parameterAt: aName).
	numberOfParameters := parameterNo.
	aParameter := aParameter at: parameterNo.
	d at: aParameter put: (tree := KeyedTree new).
	(self format: aParameter type: 'parameter' postfix: nil) cr.
^tree! !

!Accuracy methodsFor: 'running' stamp: 'WernerKassens 9/1/2014 09:24'!
tearDown
"It is the subclass' responsibility to clean up the environment after a check "
^self! !

!Accuracy methodsFor: 'private' stamp: 'WernerKassens 8/29/2014 12:37'!
tree: aTree type: aString data: aData 
aTree at: 'type' put: aString .
aTree at: 'data' put: aData.
^aTree! !

Accuracy subclass: #AccuracyTestExample
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-AccuracyFramework'!

!AccuracyTestExample commentStamp: 'WernerKassens 9/11/2014 09:38' prior: 0!
used for tests. if you want to see how to subclass Accuracy, you only need to look at initialize, initializeXXX and checkXXX, the rest is only for AccuracyTest.
if you want to see its output, just 'print it':
AccuracyTestExample new run.
!

!AccuracyTestExample methodsFor: 'tests' stamp: 'WernerKassens 9/11/2014 13:17'!
checkAaa
self argument first ifTrue:[^#(1 1)].
^{4 +(0.4 * Random new next) . 2}! !

!AccuracyTestExample methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 12:16'!
checkBbb
^self argument first size +self parameter first ! !

!AccuracyTestExample methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 22:01'!
checkCcc
^self argument first +(0.01*self parameter first).! !

!AccuracyTestExample methodsFor: 'tests' stamp: 'WernerKassens 8/31/2014 12:23'!
checkDdd
^{2 .3 }asOrderedCollection ! !

!AccuracyTestExample methodsFor: 'tests' stamp: 'WernerKassens 8/5/2014 15:29'!
checkEee
^{self parameter first .3 }! !

!AccuracyTestExample methodsFor: 'tests' stamp: 'WernerKassens 8/5/2014 15:38'!
checkFff
^#(0 0 1 0)! !

!AccuracyTestExample methodsFor: 'private' stamp: 'WernerKassens 9/1/2014 18:34'!
count
^count! !

!AccuracyTestExample methodsFor: 'initialize-release' stamp: 'WernerKassens 9/2/2014 12:30'!
initialize 
"this is always necessarily the first thing:"
super initialize .
"this is only for testSetup:"
count:=0.
"now you can set defaults:"
self result: #(#(2)#(3)).
self argument: #(#('a') #('AG')).
self parameter: #(#(1) #(3))
! !

!AccuracyTestExample methodsFor: 'initialize-release' stamp: 'WernerKassens 8/5/2014 16:50'!
initializeAaa
"this overrides defaults in #initialize:"
self result: #(#(5 3) #(4 4)).
self argument: #(#(false) #(true)).
self parameter: #(#(1 2) #(3 2.8888))! !

!AccuracyTestExample methodsFor: 'initialize-release' stamp: 'WernerKassens 9/2/2014 18:39'!
initializeCcc
"this overrides defaults in initialize"
self result: #((1)(1)(1)).
self argument: #((1)(1.1)(0.9)).! !

!AccuracyTestExample methodsFor: 'initialize-release' stamp: 'WernerKassens 8/13/2014 22:33'!
initializeDdd
"this overrides defaults in initialize"
self result: #(1.1 2.2).
self argument: #()."necessary since otherwise the default values defined in #initialize would be used"
self parameter: #().! !

!AccuracyTestExample methodsFor: 'initialize-release' stamp: 'WernerKassens 8/13/2014 22:29'!
initializeEee
"this overrides defaults in initialize"
self result: #(#(1.1 2.2) #(1 3)).
! !

!AccuracyTestExample methodsFor: 'initialize-release' stamp: 'WernerKassens 8/6/2014 16:54'!
initializeFff
"this overrides defaults in initialize"
self result: #(0 1 0 -2).
self argument: #().
self parameter:nil.! !

!AccuracyTestExample methodsFor: 'running' stamp: 'WernerKassens 9/2/2014 12:12'!
setUp 
count :=count+1.! !

!AccuracyTestExample methodsFor: 'running' stamp: 'WernerKassens 9/2/2014 12:12'!
tearDown
count :=count-(1/5).! !

!AccuracyTestExample methodsFor: 'private' stamp: 'WernerKassens 8/1/2014 15:39'!
testArgumentError
self argument: #(#(1) #(2)).! !

!AccuracyTestExample methodsFor: 'private' stamp: 'WernerKassens 8/1/2014 15:33'!
testGetterAaa
^{ self parameter .self argument.self resultsAt: 'Aaa'.self numberOfDifferentParametersAt: 'Aaa'.self numberOfDifferentResultsAt:'Aaa'}! !

!AccuracyTestExample methodsFor: 'private' stamp: 'WernerKassens 8/13/2014 17:12'!
testGetterBbb
^{ self parameter .self argument.self resultsAt: 'Bbb'.self numberOfDifferentParametersAt: 'Bbb'.self numberOfDifferentResultsAt:'Bbb'}! !

!AccuracyTestExample methodsFor: 'private' stamp: 'WernerKassens 8/1/2014 15:40'!
testParameterError
self parameter:  #(#(1) #(2)).! !

!AccuracyTestExample methodsFor: 'private' stamp: 'WernerKassens 8/1/2014 15:37'!
testResultError
self result:#(1).! !

TestCase subclass: #AccuracyTest
	instanceVariableNames: 'a dp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-AccuracyFramework'!

!AccuracyTest commentStamp: '<historical>' prior: 0!
AccuracyTest uses AccuracyTestExample!

!AccuracyTest methodsFor: 'initialize-release' stamp: 'WernerKassens 8/10/2014 16:35'!
initialize 
super initialize .
dp:=AccuracyTestExample decimalPlaces .
! !

!AccuracyTest methodsFor: 'running' stamp: 'WernerKassens 8/10/2014 16:45'!
setUp 
a:=AccuracyTestExample new.
AccuracyTestExample decimalPlaces: 3.! !

!AccuracyTest methodsFor: 'running' stamp: 'WernerKassens 8/10/2014 16:47'!
tearDown 
AccuracyTestExample decimalPlaces:dp .
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 21:57'!
testArgumentAt
	self assert: (a argumentAt: 'Aaa') = #(#(false) #(true)).
	self assert: (a argumentAt: 'Bbb') = #(#('a') #('AG')).
	self assert: (a argumentAt: 'Ddd') isNil.
	self assert: (a argumentAt: 'Ccc') = #(#(1) #(1.1) #(0.9))! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/1/2014 15:42'!
testArgumentError
self should: [a testArgumentError ]raise: Error! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/4/2014 16:41'!
testAsArray
	self assert: (a asArray: 'bla') = #('bla').
	self assert: (a asArray: #('bla'))= #('bla').
	self assert: (a asArray: #('bla')asOrderedCollection )= #('bla').! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/13/2014 18:03'!
testCalcDeviationsInMax
| r c |
c :=#(#(1 2 3)#(2 3 6) ).
r :={ (3/2) . (9/4). 4}.
	self assert: (a calcDeviations: r in: c max: false) = { (-100/3)asFloat. (-100/9)asFloat. -25}.
	self assert: (a calcDeviations: r in: c max: true) = { (100/3)asFloat. (100/3)asFloat. 50}.! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/13/2014 16:57'!
testCalcErrorOfRealResult
	self assert: (a calcErrorOf: 0.7 realResult:0.7 ) = 0.
	self assert: (a calcErrorOf: 0.7 realResult:0.0 ) = -100 .
	self assert: (a calcErrorOf: 1/4 realResult:1/2 ) = 100.
	self assert: (a calcErrorOf: 0.0 realResult:7 ) = Float infinity.
	self assert: (a calcErrorOf: 0.0 realResult:0 ) = 0.
	self assert: (a calcErrorOf: 0.0 realResult:(-1/7) ) = Float infinity negated.

! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/11/2014 09:24'!
testDataTree
	| s |
	a run.
	self assert: (a dataTree atPath: #('iterations')) = 1.
	self assert: (a dataTree atPath: #('names' 'data')) = #('Aaa' 'Bbb' 'Ccc' 'Ddd' 'Eee' 'Fff') asOrderedCollection.
	self assert: (a dataTree atPath: #('names' 'Aaa' #(1 2) #(4 4)))= (KeyedTree new
						at: 'arguments' put: true;
						at: 'data' put: #(#(1 1));
						at: 'error' put: #(300.0 300.0);
						at: 'expected result' put: #(4 4);
						at: 'result' put: #(1 1);
						at: 'type' put: 'result';
						yourself).
	self assert: (a dataTree atPath: #('names' 'Aaa' 'data')) = #(#(1 2) #(3 2.8888)).
	self assert: (a dataTree atPath: #('names' 'Bbb' #(3) 'data')) = #(#(2) #(3)).
	s:=a dataTree atPath: #('names' 'Ccc' #(3)).
	self assert: s keys size= 5.
	self assert: (a dataTree atPath: #('names' 'Fff' 'error')) = {0.(Float infinity).-100.0.(Float infinity negated)}.
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/30/2014 14:25'!
testDecimalPlaces
	self assert: a class decimalPlaces=3.
	a class decimalPlaces: 2.
	self assert: a class decimalPlaces=2.! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/31/2014 10:50'!
testExtractFromResultsWhichOnlyOne
self assert: (a extractFromResults: #( 'bla' 'blah') which: 2 onlyOne: false)='blah'.
self assert: (a testGetterAaa second) = #(true).
self assert: (a extractFromResults: #( 'bla' 'blah') which: 1 onlyOne: true)=#('bla' 'blah').
self assert: (a testGetterAaa second) = #(#(false) #(true)).
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/14/2014 08:40'!
testExtremeCollectionmax
|c|
c:=#(4 4 2).
	self assert: (a extremeCollection: c max: true) = {Float infinity negated}.
	self assert: (a extremeCollection: c max: false) = {Float infinity }.
	self assert: (a extremeCollection: {c} max: false) = {Float infinity .Float infinity .Float infinity .}! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/30/2014 11:17'!
testFormat
self assert: ((a format: #(1 'rez' 1.8899))= #('1.0' 'rez' '1.89') ).
self assert: ((a format: #(10 100 0.18899 0.018899))= #('10.0' '1.0e2' '0.189' '1.89e-2') ).
self assert:((a format: Float infinity )=#('Infinity')).
self assert:((a format: (0 - Float infinity) )=#('-Infinity'))
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 22:50'!
testFormatTypePostfix
|r|
r:=a format: #(1 'rez' 1.8899) type:'x' postfix:'%%'.
self assert: r contents= 'x: 1.0%% , rez%% , 1.89%% ' .
r:=a format: #(1.8899 true) type: '' postfix:nil.
self assert: r contents= 'x: 1.0%% , rez%% , 1.89%% : 1.89 , true ' .
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/30/2014 11:52'!
testFormatTypePostfixTree
|t|
t:=KeyedTree new.
self assert: ((a format: #(100.0 'rez' 0.18899) type:'x' postfix:'%%' tree: t) contents=  'x: 1.0e2%% , rez%% , 0.189%% ' ).
self assert:(t=(KeyedTree new at:'x' put:#(100.0 'rez' 0.18899);yourself) ).
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/1/2014 21:52'!
testGetters
|r|
r:=a testGetterAaa.
self assert:(r = #(#(#(1 2) #(3 2.8888)) #(#(false) #(true)) #(#(5 3) #(4 4)) 2 2)).
r:=a testGetterBbb.
self assert:(r = #(#(#(1) #(3)) #(#('a') #('AG')) #(#(2) #(3)) 2 2)).
 #(#(#(1) #(3)) #(#(1) #(2)) #(#(2) #(3)) 2 2)! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/1/2014 16:03'!
testIfSeveralterations
a iterations: 2.
self assert: ((a ifSeveralterations:[1])= 1 ).
a iterations: 1.
self assert: ((a ifSeveralterations:[1])= a ).
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/11/2014 09:20'!
testIterations
	| s d|
	a iterations: 2; run.
	d := a dataTree at: 'names'.
	s := (d atPath: #('Aaa' #(1 2) #(5 3) '+dev')) first negated.
	self assert: ((d atPath: #('Aaa' #(1 2) #(5 3) '-dev')) first closeTo: s).
	s := d atPath: #('Aaa' #(3 2.8888) #(5 3) 'standard deviation').
	self assert: s first > 0.
	self assert: s second = 0.
	self assert: (d atPath: #('Ddd' 'data')) = #((2 3) (2 3)).
	s:=d atPath: #('Ddd' 'error').
	self assert: (s first closeTo: -45).
	self assert: (s second closeTo: -26.6666667).! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/30/2014 18:21'!
testNumberOfDifferentParametersAt
	self assert: (a numberOfDifferentParametersAt: 'Aaa') =2.
	self assert: (a numberOfDifferentParametersAt: 'Ccc') =2.
	self assert: (a numberOfDifferentParametersAt: 'Ddd') =1.
	self assert: (a numberOfDifferentParametersAt: 'Fff') =1.
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 21:57'!
testNumberOfDifferentResultsAt
	self assert: (a numberOfDifferentResultsAt: 'Aaa') =2.
	self assert: (a numberOfDifferentResultsAt: 'Bbb') =2.
	self assert: (a numberOfDifferentResultsAt: 'Ccc') =3.
	self assert: (a numberOfDifferentResultsAt: 'Ddd') =1.
	self assert: (a numberOfDifferentResultsAt: 'Eee') =2.
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/21/2014 15:39'!
testParameterAt
	self assert: (a parameterAt: 'Aaa') = #(#(1 2) #(3 2.8888)).
	self assert: (a parameterAt: 'Bbb') = #(#(1) #(3)).
	self assert: (a parameterAt: 'Ddd') isNil.
	self assert: (a parameterAt: 'Fff') isNil! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/1/2014 15:42'!
testParameterError
self should: [a testParameterError ]raise: Error! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/10/2014 22:15'!
testPrintOn
|s|
s:= WriteStream on: String new.
a printOn: s.
self assert:(s contents ='an AccuracyTestExample ()').
s reset.
a run.
a printOn: s.
self assert: (s contents beginsWith: 'an AccuracyTestExample (
Report for: AccuracyTestExample')! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/11/2014 09:14'!
testReport
self assert:  a report = ''.
a run.
self assert: (a report beginsWith: 'Report for: AccuracyTestExample')! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/1/2014 15:41'!
testResultError
self should: [a testResultError ]raise: Error! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/11/2014 09:20'!
testResultsKeyForAtPosition
	self assert:( a resultsKeyFor: 'Aaa' AtPosition:2) = #(4 4) .
	self assert: (a resultsKeyFor: 'Bbb' AtPosition:1) = #(2). 
	a run.
	self assert: (a dataTree atPath: {'names'. 'Ccc'. #(1).a resultsKeyFor: 'Ccc' AtPosition:1.'arguments'}) = 1. 
	self assert: (a dataTree atPath: {'names'. 'Ccc'. #(1).a resultsKeyFor: 'Ccc' AtPosition:3.'arguments'}) = 0.9.
	self assert: (a resultsKeyFor: 'Eee' AtPosition:2) = #(1 3).
	self assert: (a resultsKeyFor: 'Fff' AtPosition:1) = #(0 1 0 -2) ! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 12:12'!
testSetUp
	self assert: a count = 0.
	a performCheck: 'Fff'.
	self assert: a count = (4/5).
	a iterations:3.
	a performCheck: 'Ccc'.
	self assert: a count = (16/5).! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 8/31/2014 10:12'!
testTreeTypeData
|aTree|
aTree := KeyedTree new.
self assert: (a tree: aTree type: 'aa' data:'bb') = aTree.
self assert: aTree = (KeyedTree new at:'data'put:'bb'; at: 'type'put:'aa'; yourself) .
! !

!AccuracyTest methodsFor: 'tests' stamp: 'WernerKassens 9/2/2014 12:06'!
testperformCheck
	a extractFromResults: 'bla' which: 2 onlyOne: false."for setting numberOfResults"
	self assert: (a performCheck: 'Aaa') =  #(#(1 1)).
	a extractFromResults: 'bla' which: 1 onlyOne: true."for setting numberOfResults"
	a iterations:3.
	self assert: (a performCheck: 'Fff') = #(#(0 0 1 0)#(0 0 1 0)#(0 0 1 0)).! !
Accuracy initialize!
